// Copyright (C) 2006, Ondra Kamenik

// $Id: dynare_atoms.cpp 762 2006-05-22 13:00:07Z kamenik $

#include "parser/cc/parser_exception.h"
#include "dynare_atoms.h"

#include <string>

using namespace ogdyn;
using std::string;

void DynareStaticAtoms::register_uniq(const char* name)
{
	if (varnames.query(name))
		throw ogp::ParserException(string("The name ")+name+" is not unique.", 0);
	register_name(name);
}

int DynareStaticAtoms::check_variable(const char* name) const
{
	if (0 == varnames.query(name))
		throw ogp::ParserException(std::string("Unknown name <")+name+">", 0);
	Tvarmap::const_iterator it = vars.find(name);
	if (it == vars.end())
		return -1;
	else
		return (*it).second;
}

void DynareDynamicAtoms::parse_variable(const char* in, std::string& out, int& ll) const
{
	ll = 0;
	std::string str = in;
	int left = str.find_first_of("({");
	if (left != -1) {
		out = str.substr(0, left);
		left++;
		int right = str.find_first_of(")}", left);
		if ((int)string::npos == right)
			throw ogp::ParserException(
				string("Syntax error when parsing Dynare atom <")+in+">.", 0);
		std::string tmp(str, left, right-left);
		sscanf(tmp.c_str(), "%d", &ll);
	} else {
		out = in;
	}
}

void DynareDynamicAtoms::register_uniq_endo(const char* name)
{
	if (varnames.query(name))
		throw ogp::ParserException(string("Endogenous variable <")+name+"> is not unique.",0);
	const char* ss = varnames.insert(name);
	endovars.push_back(ss);
	atom_type.insert(Tatypemap::value_type(ss, endovar));
}

void DynareDynamicAtoms::register_uniq_exo(const char* name)
{
	if (varnames.query(name))
		throw ogp::ParserException(string("Exogenous variable <")+name+"> is not unique.",0);
	const char* ss = varnames.insert(name);
	exovars.push_back(ss);
	atom_type.insert(Tatypemap::value_type(ss, exovar));
}

void DynareDynamicAtoms::register_uniq_param(const char* name)
{
	if (varnames.query(name))
		throw ogp::ParserException(string("Parameter <")+name+"> is not unique.",0);
	const char* ss = varnames.insert(name);
	params.push_back(ss);
	atom_type.insert(Tatypemap::value_type(ss, param));
}

bool DynareDynamicAtoms::is_type(const char* name, atype tp) const
{
	Tatypemap::const_iterator it = atom_type.find(name);
	if (it != atom_type.end() && (*it).second == tp)
		return true;
	else
		return false;
}

