// Copyright (C) 2006, Ondra Kamenik

// $Id: dynare_atoms.h 762 2006-05-22 13:00:07Z kamenik $

#ifndef OGDYN_DYNARE_ATOMS_H
#define OGDYN_DYNARE_ATOMS_H

#include "parser/cc/static_atoms.h"
#include "parser/cc/fine_atoms.h"

#include <map>

namespace ogdyn {

	using std::map;

	class DynareStaticAtoms : public ogp::StaticAtoms {
	public:
		DynareStaticAtoms()
			: StaticAtoms() {}
		DynareStaticAtoms(const DynareStaticAtoms& a)
			: StaticAtoms(a) {}
		virtual ~DynareStaticAtoms() {}
		/** This registers a unique varname identifier. It throws
		 * an exception if the variable name is duplicate. */
		void register_uniq(const char* name);
	protected:
		/** This returns a tree index of the given variable, and if
		 * the variable has not been registered, it throws an
		 * exception. */
		int check_variable(const char* name) const;
	};


	class DynareDynamicAtoms : public ogp::FineAtoms {
	public:
		enum atype {endovar, exovar, param};
	protected:
		typedef map<const char*, atype> Tatypemap;
		/** The map assigining a type to each name. */
		Tatypemap atom_type;
	public:
		DynareDynamicAtoms()
			: ogp::FineAtoms() {}
		DynareDynamicAtoms(const DynareDynamicAtoms& dda)
			: ogp::FineAtoms(dda) {}
		virtual ~DynareDynamicAtoms() {}
		/** This parses a variable of the forms: varname(+3),
		 * varname(3), varname, varname(-3), varname(0), varname(+0),
		 * varname(-0). */
		virtual void parse_variable(const char* in, std::string& out, int& ll) const;
		/** Registers unique name of endogenous variable. */
		void register_uniq_endo(const char* name);
		/** Registers unique name of exogenous variable. */
		void register_uniq_exo(const char* name);
		/** Registers unique name of parameter. */
		void register_uniq_param(const char* name);
		/** Return true if the name is a given type. */
		bool is_type(const char* name, atype tp) const;
	};
};

#endif

// Local Variables:
// mode:C++
// End:
